package de.cliffbreak.varo.listeners;

import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;

import com.destroystokyo.paper.Title;
import com.destroystokyo.paper.event.server.PaperServerListPingEvent;

import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.server.ServerListPingEvent;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.uitls.PacketReader;

public class PlayerJoinQuitListener implements Listener {

    private final Varo plugin;

    public PlayerJoinQuitListener(final Varo plugin) {
        this.plugin = plugin;
    }

    @EventHandler
    public void onPlayerQuit(final PlayerQuitEvent e) {
        e.setQuitMessage(null);
        if (!this.plugin.getBanUtils().isBanned(e.getPlayer().getUniqueId())) {
            this.plugin.getNPCManager().createClone(e.getPlayer());
            Bukkit.broadcast(this.plugin.getMessageUtils().getServiceMessage(e.getPlayer().getName(),
                    "hat den Server verlassen."));
        }
        this.plugin.getTeamManager().setAFK(e.getPlayer().getName());
        if (!e.getPlayer().getLocation().getWorld().getEnvironment().equals(Environment.THE_END)) {
            this.plugin.getPlayerCache().setPlayerLocationCache(e.getPlayer().getUniqueId().toString().replace("-", ""),
                    e.getPlayer().getLocation());
        }
    }

    @EventHandler
    public void onPlayerJoin(final PlayerJoinEvent e) {
        e.setJoinMessage(null);
        this.plugin.getNPCManager().syncClones(e.getPlayer());

        this.plugin.getTeamManager().setTeam(e.getPlayer());

        // Inject PacketReader
        final PacketReader packetReader = new PacketReader(this.plugin, e.getPlayer());
        packetReader.inject();

        Bukkit.broadcast(
                this.plugin.getMessageUtils().getServiceMessage(e.getPlayer().getName(), "hat den Server betreten."));

        new BukkitRunnable() {
            @Override
            public void run() {
                e.getPlayer().setResourcePack("http://cdn.cliffbreak.de/varo.zip",
                        "73046237cc843964dbe32fb9c76c0144cdbf90af");
            }
        }.runTaskLater(plugin, 5); // wait 5 ticks

        // Login cooldown + Animation
        final Player p = e.getPlayer();
        World w = null;
        for (World world : Bukkit.getWorlds()) {
            if (world.getEnvironment().equals(Environment.THE_END)) {
                w = world;
            }
        }

        p.teleport(new Location(w, 0, 0, 100));
        p.setGameMode(GameMode.SPECTATOR);
        p.setFlying(true);
        p.setFlySpeed(0.0f);
        final Title title = new Title("Varo 2020", "Bitte warte, bis das Spiel geladen hat...", 20, 8 * 20, 20);
        p.sendTitle(title);

        final BukkitTask hidePlayersTask = new BukkitRunnable() {
            @Override
            public void run() {
                for (Player player : Bukkit.getOnlinePlayers())
                    p.hidePlayer(plugin, player);
            }
        }.runTaskTimer(this.plugin, 0l, 1l);

        new BukkitRunnable() {
            @Override
            public void run() {
                plugin.getNPCManager().removeClone(e.getPlayer());
                p.setGameMode(GameMode.SURVIVAL);
                p.setFlySpeed(0.1f);
                p.setFlying(false);
                p.teleport(plugin.getPlayerCache().getPlayerLocationCache(p.getUniqueId().toString().replace("-", "")));
                hidePlayersTask.cancel();
                for (Player player : Bukkit.getOnlinePlayers())
                    p.showPlayer(plugin, player);
            }
        }.runTaskLater(this.plugin, 10 * 20);
    }

    @EventHandler
    public void onServerListPing(final ServerListPingEvent event) {
        if (event instanceof PaperServerListPingEvent) {
            handlePaperServerListPing((PaperServerListPingEvent) event);
        } else {
            this.plugin.getLogger().info("Error: Plugin is not running on a PaperServer");
        }
    }

    private void handlePaperServerListPing(final PaperServerListPingEvent e) {
        e.setHidePlayers(true);

        try {
            Instant instant = Instant.ofEpochMilli(this.plugin.getConfig().getLong("Varo.Start"));
            LocalDateTime start = instant.atZone(ZoneId.systemDefault()).toLocalDateTime();
            final LocalDateTime current = LocalDateTime.now();

            e.setMotd("§9§l" + "VARO" + "§b§l " + "Tag: " + (Duration.between(start, current).toDays() + 1) + "\n§r§7"
                    + "powered by " + "§4" + "Cliffbreak.de");
        } catch (Exception ex) {
            ex.printStackTrace();
            e.setMotd("§9§l" + "VARO" + "§b§l " + "Tag: -1" + "\n§r§7" + "powered by " + "§4" + "Cliffbreak.de");
        }
    }
}